#!/usr/bin/env python3
import sys
import matplotlib.pyplot as plt
import matplotlib
import matplotlib.cm as cm
matplotlib.rcParams['mathtext.fontset'] = 'stix'
matplotlib.rcParams['font.family'] = 'STIXGeneral'

names =  ["amulet2.1", "amulet2.0",  "amulet1.5",          "amulet1.0",    "dyposub" ,     "revsca2",    "revsca", "abc"]
labels = ["AMulet 2.1","AMulet 2.0", "AMulet 1.5", "AMulet 1.0", "DyPoSub", "RevSCA-2.0", "RevSCA", "ABC-based"]
cols =   [ "magenta", "purple",     "blue",      "green",           "orange",     "brown",  "red", "yellow"]
markers =["o",   "+",   "3", "^", "<", "p", "s", "X", "v"]
order= [1,2,3,4,5,6,7,8,9]


ypoint = range(1, 385)

for i in range(len(names)):
   index = order[i]-1
   name = names[index]
   time_file_name = "unsigned/"+name+"/"+name+".mem"
   time_file= open(time_file_name, "r")
   unsigned_times = time_file.read().split('\n')

   if name != "revsca" and name != "dyposub" :
     time_file_name = "signed/"+name+"/"+name+".mem"
     time_file= open(time_file_name, "r")
     signed_times = time_file.read().split('\n')
     times = unsigned_times[:-1] + signed_times[:-1]
   else: times = unsigned_times[:-1]

   times  = [float(i) for i in times if float(i)>-1]
   times = sorted(times)

   length=len(times)

   line,=plt.plot(times, ypoint[0:length], color=cols[index], marker = markers[index], fillstyle='none', label=labels[index])
   line.set_clip_on(False)


plt.legend(loc='upper left', bbox_to_anchor=(-0.1115, 1.32), fontsize=14, ncol=3)
plt.hlines(384, 0, 460)
plt.axis([9.3, 450,0, 400])
#plt.title("64-bit multipliers (384 instances)", fontsize=16)
plt.ylabel("Number of solved instances", fontsize=16)
plt.xlabel("Maximum Memory Usage in MB", fontsize=16)
plt.tick_params(labelsize=14)
plt.grid()
plt.xscale("log")
plt.savefig("aoki_mem.pdf", bbox_inches = 'tight', pad_inches = 0.1)
