#!/bin/sh
path="`awk '$1 == "'$SLURM_ARRAY_TASK_ID'"{print $2}' ../benchmarks`"
name="`awk '$1 == "'$SLURM_ARRAY_TASK_ID'"{print $3}' ../benchmarks`"
exec 1>"$name".log 2>"$name".err
msg () {
  echo "verify.sh: $*"
}
msg "path:     $path"
msg "name:     $name"
msg "task:     $SLURM_ARRAY_TASK_ID"
msg "host:     `hostname`"
msg "uname:    `uname -a`"
msg "start:    `date`"
msg "noturbo:  `show-turbo-mode-disabled.sh`"
msg "governor: `show-scaling-governors.sh |awk '{print $NF}'|sort|uniq -c|sed -e 's,^ *,,'`"
msg "tmp:      `df -m /tmp/|tail -1|awk '{print $(NF-1)}'`"
tmp=/tmp/verify-$$-$name
trap "rm -f $tmp*" 2 3 9 15
rewr=${tmp}_rewr.aig
miter=${tmp}_miter.cnf
rm -f $miter $rewr
msg "substitution"
time=300
space=64000

# substitution ----------------------------------------------------------

log=${name}_substitution.log
err=${name}_substitution.err
rm -f $log $err
/home/biere/bin/runlim \
  --single \
  --time-limit="$time" \
  --real-time-limit="$time" \
  --space-limit="$space" \
  /home/ritirc/algprop/src/amulet/amulet -substitute -signed $path $miter $rewr 1>$log 2>$err

# verify aig ------------------------------------------------------

log=${name}_verify_aig.log
err=${name}_verify_aig.err
rm -f $log $err
/home/biere/bin/runlim \
  --single \
  --time-limit="$time" \
  --real-time-limit="$time" \
  --space-limit="$space" \
  /home/ritirc/algprop/src/amulet/amulet -verify -signed $rewr 1>$log 2>$err

# verify cnf ------------------------------------------------------

log=${name}_verify_cnf.log
err=${name}_verify_cnf.err
rm -f $log $err
/home/biere/bin/runlim \
  --single \
  --time-limit="$time" \
  --real-time-limit="$time" \
  --space-limit="$space" \
  /home/ritirc/algprop/src/kissat/build/kissat --unsat $miter 1>$log 2>$err

rm -f $tmp*
msg "end:      `date`"
